@@ -29,7 +29,8 @@
 
 SQLITE_PRIVATE CipherParams mcAscon128Params[] =
 {
-  { "kdf_iter",          ASCON128_KDF_ITER_DEFAULT, ASCON128_KDF_ITER_DEFAULT, 1, 0x7fffffff },
+  { "kdf_iter",              ASCON128_KDF_ITER_DEFAULT, ASCON128_KDF_ITER_DEFAULT, 1, 0x7fffffff },
+  { "plaintext_header_size", 0,                         0,                         0, 100 /* restrict to db header size */ },
   CIPHER_PARAMS_SENTINEL
 };
 
@@ -42,6 +43,7 @@ SQLITE_PRIVATE CipherParams mcAscon128Params[] =
 typedef struct _ascon128Cipher
 {
   int     m_kdfIter;
+  int     m_plaintextHeaderSize;
   int     m_keyLength;
   uint8_t m_key[KEYLENGTH_ASCON128];
   uint8_t m_salt[SALTLENGTH_ASCON128];
@@ -62,6 +64,7 @@ AllocateAscon128Cipher(sqlite3* db)
   {
     CipherParams* cipherParams = sqlite3mcGetCipherParams(db, CIPHER_NAME_ASCON128);
     ascon128Cipher->m_kdfIter = sqlite3mcGetCipherParameter(cipherParams, "kdf_iter");
+    ascon128Cipher->m_plaintextHeaderSize = sqlite3mcGetCipherParameter(cipherParams, "plaintext_header_size");
   }
   return ascon128Cipher;
 }
@@ -80,6 +83,7 @@ CloneAscon128Cipher(void* cipherTo, void* cipherFrom)
   Ascon128Cipher* ascon128CipherTo = (Ascon128Cipher*) cipherTo;
   Ascon128Cipher* ascon128CipherFrom = (Ascon128Cipher*) cipherFrom;
   ascon128CipherTo->m_kdfIter = ascon128CipherFrom->m_kdfIter;
+  ascon128CipherTo->m_plaintextHeaderSize = ascon128CipherFrom->m_plaintextHeaderSize;
   ascon128CipherTo->m_keyLength = ascon128CipherFrom->m_keyLength;
   memcpy(ascon128CipherTo->m_key, ascon128CipherFrom->m_key, KEYLENGTH_ASCON128);
   memcpy(ascon128CipherTo->m_salt, ascon128CipherFrom->m_salt, SALTLENGTH_ASCON128);
@@ -117,7 +121,6 @@ static void
 GenerateKeyAscon128Cipher(void* cipher, char* userPassword, int passwordLength, int rekey, unsigned char* cipherSalt)
 {
   Ascon128Cipher* ascon128Cipher = (Ascon128Cipher*) cipher;
-  int bypass = 0;
 
   int keyOnly = 1;
   if (rekey || cipherSalt == NULL)
@@ -130,52 +133,10 @@ GenerateKeyAscon128Cipher(void* cipher, char* userPassword, int passwordLength,
     memcpy(ascon128Cipher->m_salt, cipherSalt, SALTLENGTH_ASCON128);
   }
 
-  /* Bypass key derivation if the key string starts with "raw:" */
-  if (passwordLength > 4 && !memcmp(userPassword, "raw:", 4))
-  {
-    const int nRaw = passwordLength - 4;
-    const unsigned char* zRaw = (const unsigned char*) userPassword + 4;
-    switch (nRaw)
-    {
-      /* Binary key (and salt) */
-      case KEYLENGTH_ASCON128 + SALTLENGTH_ASCON128:
-        if (!keyOnly)
-        {
-          memcpy(ascon128Cipher->m_salt, zRaw + KEYLENGTH_ASCON128, SALTLENGTH_ASCON128);
-        }
-        /* fall-through */
-      case KEYLENGTH_ASCON128:
-        memcpy(ascon128Cipher->m_key, zRaw, KEYLENGTH_ASCON128);
-        bypass = 1;
-        break;
-
-      /* Hex-encoded key */
-      case 2 * KEYLENGTH_ASCON128:
-        if (sqlite3mcIsHexKey(zRaw, nRaw) != 0)
-        {
-          sqlite3mcConvertHex2Bin(zRaw, nRaw, ascon128Cipher->m_key);
-          bypass = 1;
-        }
-        break;
-
-      /* Hex-encoded key and salt */
-      case 2 * (KEYLENGTH_ASCON128 + SALTLENGTH_ASCON128):
-        if (sqlite3mcIsHexKey(zRaw, nRaw) != 0)
-        {
-          sqlite3mcConvertHex2Bin(zRaw, 2 * KEYLENGTH_ASCON128, ascon128Cipher->m_key);
-          if (!keyOnly)
-          {
-            sqlite3mcConvertHex2Bin(zRaw + 2 * KEYLENGTH_ASCON128, 2 * SALTLENGTH_ASCON128, ascon128Cipher->m_salt);
-          }
-          bypass = 1;
-        }
-        break;
-
-      default:
-        break;
-    }
-  }
-
+  /* Bypass key derivation, if raw key (and optionally salt) are given */
+  int bypass = sqlite3mcExtractRawKey(userPassword, passwordLength,
+                                      keyOnly, KEYLENGTH_ASCON128, SALTLENGTH_ASCON128,
+                                      ascon128Cipher->m_key, ascon128Cipher->m_salt);
   if (!bypass)
   {
     ascon_pbkdf2(ascon128Cipher->m_key, KEYLENGTH_ASCON128,
@@ -210,10 +171,26 @@ EncryptPageAscon128Cipher(void* cipher, int page, unsigned char* data, int len,
   int nReserved = (reserved == 0) ? 0 : GetReservedAscon128Cipher(cipher);
   int n = len - nReserved;
   uint64_t mlen = n;
+  int usePlaintextHeader = 0;
 
   /* Generate one-time keys */
   uint8_t otk[ASCON_HASH_BYTES];
-  int offset;
+  int offset = 0;
+
+  /* Check whether a plaintext header should be used */
+  if (page == 1)
+  {
+    int plaintextHeaderSize = ascon128Cipher->m_plaintextHeaderSize;
+    if (plaintextHeaderSize > 0)
+    {
+      usePlaintextHeader = 1;
+      offset = (plaintextHeaderSize > CIPHER_PAGE1_OFFSET) ? plaintextHeaderSize : CIPHER_PAGE1_OFFSET;
+    }
+    else
+    {
+      offset = CIPHER_PAGE1_OFFSET;
+    }
+  }
 
   /* Check whether number of required reserved bytes and actually reserved bytes match */
   if (nReserved > reserved)
@@ -229,11 +206,10 @@ EncryptPageAscon128Cipher(void* cipher, int page, unsigned char* data, int len,
     chacha20_rng(data + n + PAGE_TAG_LEN_ASCON128, PAGE_NONCE_LEN_ASCON128);
     AsconGenOtk(otk, ascon128Cipher->m_key, data + n + PAGE_TAG_LEN_ASCON128, page);
 
-    offset = (page == 1) ? CIPHER_PAGE1_OFFSET : 0;
     ascon_aead_encrypt(data + offset, data + n, data + offset, mlen - offset,
                        NULL /* ad */, 0 /* adlen*/,
                        data + n + PAGE_TAG_LEN_ASCON128, otk);
-    if (page == 1)
+    if (page == 1 && usePlaintextHeader == 0)
     {
       memcpy(data, ascon128Cipher->m_salt, SALTLENGTH_ASCON128);
     }
@@ -249,11 +225,10 @@ EncryptPageAscon128Cipher(void* cipher, int page, unsigned char* data, int len,
     AsconGenOtk(otk, ascon128Cipher->m_key, nonce, page);
 
     /* Encrypt */
-    offset = (page == 1) ? CIPHER_PAGE1_OFFSET : 0;
     ascon_aead_encrypt(data + offset, dummyTag, data + offset, mlen - offset,
                        NULL /* ad */, 0 /* adlen*/,
                        nonce, otk);
-      if (page == 1)
+      if (page == 1 && usePlaintextHeader == 0)
     {
       memcpy(data, ascon128Cipher->m_salt, SALTLENGTH_ASCON128);
     }
@@ -271,10 +246,26 @@ DecryptPageAscon128Cipher(void* cipher, int page, unsigned char* data, int len,
   int n = len - nReserved;
   uint64_t clen = n;
   int tagOk;
+  int usePlaintextHeader = 0;
 
   /* Generate one-time keys */
   uint8_t otk[ASCON_HASH_BYTES];
-  int offset;
+  int offset = 0;
+
+  /* Check whether a plaintext header should be used */
+  if (page == 1)
+  {
+    int plaintextHeaderSize = ascon128Cipher->m_plaintextHeaderSize;
+    if (plaintextHeaderSize > 0)
+    {
+      usePlaintextHeader = 1;
+      offset = (plaintextHeaderSize > CIPHER_PAGE1_OFFSET) ? plaintextHeaderSize : CIPHER_PAGE1_OFFSET;
+    }
+    else
+    {
+      offset = CIPHER_PAGE1_OFFSET;
+    }
+  }
 
   /* Check whether number of required reserved bytes and actually reserved bytes match */
   if (nReserved > reserved)
@@ -289,7 +280,6 @@ DecryptPageAscon128Cipher(void* cipher, int page, unsigned char* data, int len,
     AsconGenOtk(otk, ascon128Cipher->m_key, data + n + PAGE_TAG_LEN_ASCON128, page);
 
     /* Determine MAC and decrypt */
-    offset = (page == 1) ? CIPHER_PAGE1_OFFSET : 0;
     tagOk = ascon_aead_decrypt(data + offset, data + offset, clen - offset,
                                NULL /* ad */, 0 /* adlen */,
                                data + n, data + n + PAGE_TAG_LEN_ASCON128, otk);
@@ -310,7 +300,7 @@ DecryptPageAscon128Cipher(void* cipher, int page, unsigned char* data, int len,
         rc = (page == 1) ? SQLITE_NOTADB : SQLITE_CORRUPT;
       }
     }
-    if (page == 1 && rc == SQLITE_OK)
+    if (page == 1 && usePlaintextHeader == 0 && rc == SQLITE_OK)
     {
       memcpy(data, SQLITE_FILE_HEADER, 16);
     }
@@ -326,11 +316,10 @@ DecryptPageAscon128Cipher(void* cipher, int page, unsigned char* data, int len,
     AsconGenOtk(otk, ascon128Cipher->m_key, nonce, page);
 
     /* Decrypt */
-    offset = (page == 1) ? CIPHER_PAGE1_OFFSET : 0;
     tagOk = ascon_aead_decrypt(data + offset, data + offset, clen - offset,
                                NULL /* ad */, 0 /* adlen */,
                                dummyTag, nonce, otk);
-    if (page == 1)
+    if (page == 1 && usePlaintextHeader == 0)
     {
       memcpy(data, SQLITE_FILE_HEADER, 16);
     }
